/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2014                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/

/**
 * \file CubePostDerivedMetric.h
 * \brief Defines a metric, which calculates its value using user defined expression in CubePL"
 */
#ifndef CUBE_POSTDERIVED_METRIC_H
#define CUBE_POSTDERIVED_METRIC_H


#include <iosfwd>
#include <map>

// #include "Matrix.h"
#include "CubeVertex.h"
#include "CubeCnode.h"
#include "CubeThread.h"
#include "CubeMetric.h"
// #include "CubeAdvMatrix.h"
// #include "CubeAdvancedMatrix.h"
// #include "CubeMemoryMatrix.h"
// #include "CubeIDdeliverer.h"
// #include "CubeDeepSearchEnumerator.h"
#include "CubeValues.h"

/*
   *----------------------------------------------------------------------------
 *
 * class Metric
 *
 ***************************----------------------------------------------------------------------------
 */

namespace cube
{
/*  class Cnode;
   class Thread;*/

/**
 * Metric has a treelike structure and is a kind of Vertex.
 */

class PostDerivedMetric : public Metric
{
public:

    PostDerivedMetric( const std::string& disp_name,
                       const std::string& uniq_name,
                       const std::string& dtype,
                       const std::string& uom,
                       const std::string& val,
                       const std::string& url,
                       const std::string& descr,
                       FileFinder*        ffinder,
                       Metric*            parent,
                       uint32_t           id = 0,
                       const std::string  _expression = "",
                       const std::string& _expression_init = "",
                       TypeOfMetric       is_ghost = CUBE_METRIC_NO_GHOST
                       )
        : Metric( disp_name, uniq_name, dtype, uom, val, url, descr, ffinder, parent, id, _expression, _expression_init,  is_ghost )
    {
        adv_sev_mat = NULL;
        initialized = true; // with creation initialisation is done
        cache       = new SimpleCache( ncid, ntid, CUBE_CALCULATE_EXCLUSIVE );
    }
    virtual
    ~PostDerivedMetric();

    virtual row_of_objects_t*
    create_calltree_id_maps( IDdeliverer*      _id,
                             Cnode*            cnode,
                             row_of_objects_t* row )
    {
        return row;
    };

    virtual TypeOfMetric
    get_type_of_metric() const
    {
        return CUBE_METRIC_POSTDERIVED;
    };
    virtual std::string
    get_metric_kind() const
    {
        return "POSTDERIVED";
    };


    using cube::Metric::get_sev_adv;

    virtual Value*
    get_sev_adv( Cnode*             cnode,
                 CalculationFlavour cnf );

    virtual Value*
    get_sev_adv( Cnode*             cnode,
                 CalculationFlavour cfv,
                 Sysres*            sys,
                 CalculationFlavour sf );




    virtual IndexFormat
    get_index_format()
    {
        return CUBE_INDEX_FORMAT_SPARSE;
    };



    virtual
    void
    create_data_container( uint32_t, uint32_t )
    {
    };                                                                           ///< Empty hook
    virtual
    void
    create_existing_data_container( uint32_t, uint32_t )
    {
    };                                                                           ///< Empty hook
};
}
#endif
